# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'GlacierBackup'
        db.create_table('icybackup_glacierbackup', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('glacier_id', self.gf('django.db.models.fields.CharField')(unique=True, max_length=138)),
            ('date', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal('icybackup', ['GlacierBackup'])


    def backwards(self, orm):
        # Deleting model 'GlacierBackup'
        db.delete_table('icybackup_glacierbackup')


    models = {
        'icybackup.glacierbackup': {
            'Meta': {'object_name': 'GlacierBackup'},
            'date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'glacier_id': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '138'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        }
    }

    complete_apps = ['icybackup']


import os, sys, time, shutil
from optparse import make_option
from tempfile import mkdtemp, NamedTemporaryFile
import tarfile
from boto.glacier.layer2 import Layer2 as Glacier
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from distutils.dir_util import copy_tree

from ...components import db, lib

# Based on: http://code.google.com/p/django-backup/
# Based on: http://www.djangosnippets.org/snippets/823/
# Based on: http://www.yashh.com/blog/2008/sep/05/django-database-backup-view/
class Command(BaseCommand):
	option_list = BaseCommand.option_list + (
		make_option('-i', '--file', default=None, dest='input',
			help='Read backup from file'),
		make_option('-I', '--stdin', action='store_true', dest='stdin',
			help='Read backup from standard input'),
		)
	help = "Restore a Django installation (database and media directory)."

	def handle(self, *args, **options):
		extras = options.get('extras')

		input_file = options.get('input')
		input_from_stdin = options.get('stdin')
		input_file_temporary = False
		
		if input_file is None and input_from_stdin is None:
			raise CommandError('You must specify an input file')

		media_root = settings.MEDIA_ROOT
		
		# read from stdin
		if input_from_stdin:
			input_file_temporary = True
			input_file_obj = NamedTemporaryFile(delete=False)
			input_file_obj.write(sys.stdin.read())
			input_file_obj.close()
			input_file = input_file_obj.name

		# Create a temporary directory to extract our backup to
		extract_root = mkdtemp()
		backup_root = os.path.join(extract_root, 'backup')	
		database_root = os.path.join(backup_root, 'databases')
		
		# extract the gzipped tarball
		with tarfile.open(input_file, 'r') as tf:
			tf.extractall(extract_root)

		# Restore databases
		db.restore_from(settings, database_root)
		
		# Restore media directory
		copy_tree(os.path.join(backup_root, 'media'), media_root)
		
		# clean up
		lib.rm_rf(extract_root)
		if input_file_temporary:
			os.unlink(input_file)



from .. import models
from boto.glacier.layer2 import Layer2 as Glacier

# upload to amazon glacier
def upload(glacier_vault, output_file, settings):
	g = Glacier(aws_access_key_id=settings.AWS_ACCESS_KEY_ID, aws_secret_access_key=settings.AWS_SECRET_ACCESS_KEY)
	for i in g.list_vaults():
		if glacier_vault == i.arn:
			vault = i
			break
	else:
		raise CommandError('The specified vault could not be accessed.')
	id = vault.upload_archive(output_file)
	
	# record backup internally
	# we don't need this record in order to restore from backup (obviously!)
	# but it makes pruning the backup set easier, and amazon reccomends it
	record = models.GlacierBackup.objects.create(glacier_id=id)
	record.save()


from django.db import models

class GlacierBackup (models.Model):
	glacier_id = models.CharField(max_length=138, unique=True, verbose_name="Glacier backup ID")
	date = models.DateTimeField(auto_now_add=True)
	class Meta:
		verbose_name = "Glacier backup"
		verbose_name_plural = "Glacier backups"


import os
from django.core.management.base import CommandError
from tempfile import mkstemp
from subprocess import check_call
from shutil import copy
from django.conf import settings

BACKUP = 1
RESTORE = 2

MYSQL_BIN = settings.get('MYSQL_BIN', 'mysql')
MYSQLDUMP_BIN = settings.get('MYSQLDUMP_BIN', 'mysqldump')
PG_DUMP_BIN = settings.get('PG_DUMP_BIN', 'pg_dump')
PG_RESTORE_BIN = settings.get('PG_RESTORE_BIN', 'pg_restore')

def _database_dict_from_settings(settings):
    if hasattr(settings, 'DATABASES'):
        database_list = settings.DATABASES
    else:
        # database details are in the old format, so convert to the new one
        database_list = {
            'default': {
                'ENGINE': settings.DATABASE_ENGINE,
                'NAME': settings.DATABASE_NAME,
                'USER': settings.DATABASE_USER,
                'PASSWORD': settings.DATABASE_PASSWORD,
                'HOST': settings.DATABASE_HOST,
                'PORT': settings.DATABASE_PORT,
            }
        }
    return database_list

def backup_to(settings, dir):
    for name, database in _database_dict_from_settings(settings).iteritems():
        do(BACKUP, database, os.path.join(dir, name))

def restore_from(settings, dir):
    for name, database in _database_dict_from_settings(settings).iteritems():
        do(RESTORE, database, os.path.join(dir, name))

def do(action, database, f):
    engine = database['ENGINE']
    if 'mysql' in engine:
        __mysql(action, database, f)
    elif 'postgresql' in engine:
        __postgresql(action, database, f)
    elif 'sqlite3' in engine:
        __sqlite(action, database, f)
    else:
        raise CommandError('{} in {} engine not implemented'.format('Backup' if action == BACKUP else 'Restore', engine))

def __sqlite(action, database, f):
    if action == BACKUP:
        copy(database['NAME'], f)
    elif action == RESTORE:
        copy(f, database['NAME'])

def __mysql(action, database, f):
    if action == BACKUP:
        command = [MYSQLDUMP_BIN]
    elif action == RESTORE:
        command = [MYSQL_BIN]
    
    if 'USER' in database:
        command += ["--user=%s" % database['USER']]
    if 'PASSWORD' in database:
        command += ["--password=%s" % database['PASSWORD']]
    if 'HOST' in database:
        command += ["--host=%s" % database['HOST']]
    if 'PORT' in database:
        command += ["--port=%s" % database['PORT']]
    command += [database['NAME']]
    
    if action == BACKUP:
        with open(f, 'w') as fo:
            check_call(command, stdout=fo)
    elif action == RESTORE:
        with open(f, 'r') as fo:
            check_call(command, stdin=fo)

def __postgresql(action, database, f):
    if action == BACKUP:
        command = [PG_DUMP_BIN, '--format=c']
    elif action == RESTORE:
        command = [PG_RESTORE_BIN, '-Oxc']
    
    if 'USER' in database and database['USER']:
        command.append("--username={}".format(database['USER']))
    if 'HOST' in database and database['HOST']:
        command.append("--host={}".format(database['HOST']))
    if 'PORT' in database and database['PORT']:
        command.append("--port={}".format(database['PORT']))
    if 'NAME' in database and database['NAME']:
        if action == RESTORE:
            command.append('--dbname={}'.format(database['NAME']))
        else:
            command.append(database['NAME'])
    
    if 'PASSWORD' in database and database['PASSWORD']:
        # create a pgpass file that always returns the same password, as a secure temp file
        password_fd, password_path = mkstemp()
        password_file = os.fdopen(password_fd, 'w')
        password_file.write('*:*:*:*:{}'.format(database['PASSWORD']))
        password_file.close()
        os.environ['PGPASSFILE'] = password_path
    else:
        command.append('-w')
    
    if action == BACKUP:
        with open(f, 'w') as fo:
            check_call(command, stdout=fo)
    elif action == RESTORE:
        with open(f, 'r') as fo:
            check_call(command, stdin=fo)
        
    # clean up
    if 'PASSWORD' in database and database['PASSWORD']:
        os.remove(password_path)



from django.contrib import admin
from .models import GlacierBackup

class GlacierBackupAdmin (admin.ModelAdmin):
	list_display = ('date', 'glacier_id')
	readonly_fields = ('glacier_id', 'date')
	
	def has_add_permission(self, request):
		return False
		
	def has_delete_permission(self, request, obj=None):
		return False

admin.site.register(GlacierBackup, GlacierBackupAdmin)


import os, sys, time
from optparse import make_option
from tempfile import mkdtemp, NamedTemporaryFile
import tarfile
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError

from ...components import db, glacier, lib

# Based on: http://code.google.com/p/django-backup/
# Based on: http://www.djangosnippets.org/snippets/823/
# Based on: http://www.yashh.com/blog/2008/sep/05/django-database-backup-view/
class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option('-o', '--output', default=None, dest='output',
            help='Write backup to file'),
        make_option('-d', '--outdir', default=None, dest='outdir',
            help='Write backup to timestamped file in a directory'),
        make_option('-g', '--glacier', default=None, dest='glacier',
            help='Upload backup to the Amazon Glacier vault with the given ARN'),
        make_option('-O', '--stdout', action='store_true', dest='stdout',
            help='Output backup tarball to standard output'),
        make_option('--extra', '-e', action='append', default=[], dest='extras',
            help='Include extra directories or files in the backup tarball'),
    )
    help = "Back up a Django installation (database and media directory)."

    def handle(self, *args, **options):
        extras = options.get('extras')
        
        output_file = options.get('output')
        output_dir = options.get('outdir')
        glacier_vault = options.get('glacier')
        output_to_stdout = options.get('stdout')
        output_file_temporary = False
        
        # glacier backups go to a temporary file
        if glacier_vault is not None or output_to_stdout:
            output_file_temporary = True
            output_file_obj = NamedTemporaryFile(delete=False)
            output_file_obj.close() # we'll open it later
            output_file = output_file_obj.name
        
        if output_file is None:
            if output_dir is None:
                raise CommandError('You must specify an output file')
            else:
                output_file = os.path.join(output_dir, '{}.tgz'.format(_time()))
            
        media_root = settings.MEDIA_ROOT
        
        # Create a temporary directory to perform our backup in
        backup_root = mkdtemp()
        database_root = os.path.join(backup_root, 'databases')
        os.mkdir(database_root)

        # Back up databases
        db.backup_to(settings, database_root)
        
        # create backup gzipped tarball
        with tarfile.open(output_file, 'w:gz') as tf:
            tf.add(database_root, arcname='backup/databases')
            tf.add(media_root, arcname='backup/media')
            if len(extras) > 0:
                extras_mf = NamedTemporaryFile(delete=False)
                for count, extra in enumerate(extras):
                    tf.add(extra, arcname='backup/extras/{}'.format(count))
                    extras_mf.write('{},{}\n'.format(count, extra.replace(',','\\,')))
                extras_mf.close()
                tf.add(extras_mf.name, arcname='backup/extras/manifest')
                os.unlink(extras_mf.name)
        
        # upload to glacier
        if glacier_vault is not None:
            glacier.upload(glacier_vault, output_file, settings)
        
        # output to stdout
        if output_to_stdout:
            with open(output_file, 'r') as f:
                sys.stdout.write(f.read())
        
        # clean up
        lib.rm_rf(backup_root)
        if output_file_temporary:
            os.unlink(output_file)

def _time():
    return time.strftime('%Y%m%d-%H%M%S')



from setuptools import setup, find_packages
import subprocess
from icybackup import __version__

def get_long_desc():
    """Use Pandoc to convert the readme to ReST for the PyPI."""
    try:
        return subprocess.check_output(['pandoc', '-f', 'markdown', '-t', 'rst', 'README.mdown'])
    except:
        print "WARNING: The long readme wasn't converted properly"

setup(name='django-icybackup',
    version=__version__,
    description='A Django database/media backup tool with Amazon Glacier and local folder support',
    long_description=get_long_desc(),
    author='Adam Brenecki, St Barnabas\' Theological College',
    author_email='abrenecki@sbtc.org.au',
    url='https://github.com/stbarnabas/django-icybackup',
    packages=find_packages(),
    include_package_data=True,
    setup_requires=[
        'setuptools_git>=0.3',
    ],
    install_requires=[
        'boto',
    ],
)
